from typing import Optional, Union, Dict, Any, List

from reflex import Component, Var

from ..base import AntdComponent, ContainVar, JsValue, ReactNode
from ..constant import TypeType


class Modal(AntdComponent):
    tag = 'Modal'

    class_names: Optional[Var[ContainVar]]
    styles: Optional[Var[ContainVar]]
    cancel_button_props: Optional[Var[ContainVar]]
    cancel_text: Optional[Var[ReactNode]]
    centered: Optional[Var[bool]]
    close_icon: Optional[Var[Union[ReactNode, bool]]]
    confirm_loading: Optional[Var[bool]]
    destroy_on_close: Optional[Var[bool]]
    focus_trigger_after_close: Optional[Var[bool]]
    footer: Optional[Var[ContainVar]]
    force_render: Optional[Var[bool]]
    get_container: Optional[Var[Union[bool, ContainVar]]]
    keyboard: Optional[Var[bool]]
    mask: Optional[Var[bool]]
    mask_closable: Optional[Var[bool]]
    modal_render: Optional[Var[ReactNode]]
    ok_button_props: Optional[Var[ContainVar]]
    ok_text: Optional[Var[ReactNode]]
    ok_type: Optional[Var[TypeType]]
    title: Optional[Var[ReactNode]]
    open: Optional[Var[bool]]
    width: Optional[Var[Union[str, int]]]
    wrapClassName: Optional[Var[str]]
    zIndex: Optional[Var[int]]

    def get_event_triggers(self) -> Dict[str, Any]:
        _triggers = super().get_event_triggers()
        _triggers.update({
            "after_close": lambda: [],
            "on_cancel": lambda event: [],
            "on_ok": lambda event: [],
            "after_open_change": lambda open: [],
        })
        return _triggers


modal = Modal.create



from typing import Optional, Union, Dict, Any, List
from reflex import Var, Component
from reflex.constants import EventTriggers

from ..base import AntdComponent, ContainVar, JsValue, ReactNode
from ..constant import AlignType, DirectionType, SizeType, VariantType


class Form(AntdComponent):
    tag = 'Form'

    colon: Optional[Var[bool]]
    disabled: Optional[Var[bool]]
    fields: Optional[Var[ContainVar]]
    initial_values: Optional[Var[Dict]]
    label_align: Optional[Var[AlignType]]
    label_wrap: Optional[Var[bool]]
    label_col: Optional[Var[Dict]]
    layout: Optional[Var[DirectionType]]
    name: Optional[Var[str]]
    preserve: Optional[Var[bool]]
    required_mark: Optional[Var[Union[bool, JsValue]]]
    scrollTo_first_error: Optional[Var[Union[bool, Dict]]]
    size: Optional[Var[SizeType]]
    validate_messages: Optional[Var[Dict]]
    validate_trigger: Optional[Var[Union[str, List[str]]]]
    variant: Optional[Var[VariantType]]
    wrapper_col: Optional[Var[Dict]]

    def get_event_triggers(self) -> Dict[str, Any]:
        _triggers = super().get_event_triggers()
        _triggers.update({
            "on_fields_change": lambda changed, all: [changed],
            'on_finish': lambda values: [values],
            "on_finish_failed": lambda ev: [ev.values, ev.errorFields, ev.outOfDate],
            "on_values_change": lambda changed, all: [changed, all],
        })
        return _triggers


class FormItem(AntdComponent):
    tag = 'Form.Item'

    colon: Optional[Var[bool]]
    dependencies: Optional[Var[List[Union[str, int, List[Union[str, int]]]]]]
    extra: Optional[Var[ReactNode]]
    getValue_from_event: Optional[Var[JsValue]]
    get_value_props: Optional[Var[JsValue]]
    has_feedback: Optional[Var[Union[bool, ContainVar]]]
    help: Optional[Var[ReactNode]]
    hidden: Optional[Var[bool]]
    html_for: Optional[Var[str]]
    initial_value: Optional[Var[str]]
    label: Optional[Var[str]]
    label_align: Optional[Var[AlignType]]
    label_col: Optional[Var[Dict]]
    message_variables: Optional[Var[JsValue]]
    name: Optional[Var[Union[str, int, List[Union[str, int]]]]]
    normalize: Optional[Var[JsValue]]
    no_style: Optional[Var[bool]]
    preserve: Optional[Var[bool]]
    required: Optional[Var[bool]]
    rules: Optional[Var[Union[List[Dict], ContainVar]]]
    should_update: Optional[Var[bool]]
    tooltip: Optional[Var[Union[Component, ContainVar]]]
    trigger: Optional[Var[str]]
    validate_debounce: Optional[Var[float]]
    validate_first: Optional[Var[Union[bool, str]]]
    validate_status: Optional[Var[str]]
    validate_trigger: Optional[Var[Union[str, List[str]]]]
    value_prop_name: Optional[Var[str]]
    wrapper_col: Optional[Var[Dict]]

    rules: Optional[Var[List[Dict]]]


class FormList(AntdComponent):
    tag = 'Form.List'

    # children: Optional[Var[JsValue]]
    initial_value: Optional[Var[List]]
    name: Optional[Var[Union[str, int, List[Union[str, int]]]]]
    rules: Optional[Var[ContainVar]]


class FormProvider(AntdComponent):
    tag = 'Form.Provider'

    def get_event_triggers(self) -> Dict[str, Any]:
        _triggers = super().get_event_triggers()
        _triggers.update({
            "on_form_change": lambda form_name, info: [form_name, info],
            'on_form_finish': lambda form_name, info: [form_name, info],
        })
        return _triggers


form = Form.create
form_item = FormItem.create
form_list = FormList.create
form_provider = FormProvider.create



from typing import Optional, Union, Dict, Any

from reflex import Component, Var
from reflex.utils import imports
from reflex.constants import EventTriggers

from ..base import AntdComponent, AntdSubComponent, ReactNode, ContainVar
from ..constant import BreakpointType, ThemeType


class LayoutBase(AntdComponent):
    pass


class Layout(LayoutBase):
    tag = 'Layout'

    has_sider: Optional[Var[bool]]

    # def _get_custom_code(self) -> str | None:
    #     return """
    #     const { Header, Content, Footer, Sider } = Layout;
    #     """


class SubLayout(AntdComponent):
    width: Optional[Var[Union[int, str]]]


class Header(SubLayout):
    tag = 'Layout.Header'


class Content(SubLayout):
    tag = 'Layout.Content'


class Footer(SubLayout):
    tag = 'Layout.Footer'


class Sider(SubLayout):
    tag = 'Layout.Sider'

    breakpoint: Optional[Var[Union[BreakpointType, Dict]]]
    collapsed: Optional[Var[bool]]
    collapsed_width: Optional[Var[int]]
    collapsible: Optional[Var[bool]]
    default_collapsed: Optional[Var[bool]]
    reverse_arrow: Optional[Var[bool]]
    theme: Optional[Var[ThemeType]]
    trigger: Optional[Var[ReactNode]]
    zero_width_trigger_style: Optional[Var[Union[Dict, ContainVar]]]

    def get_event_triggers(self) -> Dict[str, Any]:
        _triggers = super().get_event_triggers()
        _triggers.update({
            'on_breakpoint': lambda broken: [broken],
            'on_collapse': lambda collapsed, type: [collapsed, type],
        })
        return _triggers


layout = Layout.create
header = Header.create
content = Content.create
footer = Footer.create
sider = Sider.create

